module.exports = bootstrap

var couchdbBootstrap = require('couchdb-bootstrap')

function bootstrap (state, next) {
  couchdbBootstrap(state.adminCouch, state.bootstrapPath, {
    mapDbName: {
      'raw-data': state.config.db.rawData,
      'integrated-data': state.config.db.integratedData
    }
  }, function (error, response) {
    if (error) {
      return next(error)
    }

    if ('configure' in response && response.configure && response.configure.length) {
      state.log([state.name, 'bootstrap', 'info'], 'config updated')
    }

    if ('secure' in response) {
      Object.keys(response.secure).forEach(function (dbname) {
        state.log([state.name, 'bootstrap', 'info'], '_security set on database ' + dbname + '.')
      })
    }

    if ('push' in response) {
      Object.keys(response.push).forEach(function (dbname) {
        if (dbname === '_users') {
          var usernames = response.push._users.map(function (resp) {
            return resp.id.split(':')[1]
          })

          return state.log([state.name, 'bootstrap', 'info'], 'users created or updated: ' + usernames.join(', '))
        }

        var counts = response.push[dbname].reduce(function (memo, resp) {
          if (resp.id.match(/^_design/)) {
            memo.ddocs++
          } else {
            memo.docs++
          }
          return memo
        }, {
          ddocs: 0,
          docs: 0
        })

        if (counts.ddocs) {
          state.log([state.name, 'bootstrap', 'info'], '_design docs added or updated on database ' + dbname + ': ' + counts.ddocs + '.')
        }
        if (counts.docs) {
          state.log([state.name, 'bootstrap', 'info'], 'docs added or updated on database ' + dbname + ': ' + counts.docs + '.')
        }
      })
    }

    next(null)
  })
}
